@@ -30,6 +30,7 @@
 #include <QtSql/QSqlQuery>
 #include <QtXml/QDomDocument>
 #include <QtXml/QDomElement>
+#include <QtSql/QSqlError>
 
 DbInitializer::DbInitializer( const QSqlDatabase &database, const QString &templateFile )
   : mDatabase( database ), mTemplateFile( templateFile )
@@ -103,9 +104,23 @@ bool DbInitializer::checkTable( const QDomElement &element )
       ColumnEntry entry;
       entry.first = columnElement.attribute( QLatin1String("name") );
       if ( columnElement.attribute( QLatin1String("sqltype") ).isEmpty() )
-        entry.second = sqlType( columnElement.attribute( QLatin1String("type") ) ) + QLatin1String( " " ) + columnElement.attribute( QLatin1String("properties") );
+        entry.second = sqlType( columnElement.attribute( QLatin1String("type") ) );
       else
-        entry.second = columnElement.attribute( QLatin1String("sqltype") ) + QLatin1String( " " ) + columnElement.attribute( QLatin1String("properties") );
+        entry.second = columnElement.attribute( QLatin1String("sqltype") ); 
+      QString props = columnElement.attribute(QLatin1String("properties"));
+
+      // TODO: we need a nicer way for this...
+      // special cases for sqlite
+      if ( mDatabase.driverName().startsWith( QLatin1String("QSQLITE") ) ) {
+        if ( props.contains(QLatin1String("PRIMARY KEY")) && entry.second == QLatin1String("BIGINT") )
+            entry.second = QLatin1String("INTEGER");
+        if ( props.contains(QLatin1String("BINARY")) && !props.contains(QLatin1String("COLLATE BINARY")) )
+          props.replace(QLatin1String("BINARY"), QLatin1String("COLLATE BINARY"));
+        if ( props.contains(QLatin1String("character set utf8 collate utf8_bin")) )
+          props.remove(QLatin1String("character set utf8 collate utf8_bin"));
+      }
+      entry.second += QLatin1String(" ") + props;
+      // special cases for PostgreSQL
       if ( mDatabase.driverName() == QLatin1String( "QPSQL" ) ) {
         if ( entry.second.contains( QLatin1String("AUTOINCREMENT") ) )
           entry.second = QLatin1String("SERIAL PRIMARY KEY NOT NULL");
@@ -113,12 +128,29 @@ bool DbInitializer::checkTable( const QDomElement &element )
           entry.second = QLatin1String("BYTEA");
         if ( entry.second.startsWith( QLatin1String("CHAR") ) )
           entry.second.replace(QLatin1String("CHAR"), QLatin1String("VARCHAR"));
-      } else if ( mDatabase.driverName().startsWith( QLatin1String("QMYSQL") ) ) {
+        if ( entry.second.contains( QLatin1String( "BINARY" ) ) )
+          entry.second.remove( QLatin1String( "BINARY" ) );
+        if ( entry.second.contains(QLatin1String("character set utf8 collate utf8_bin")) )
+          entry.second.remove(QLatin1String("character set utf8 collate utf8_bin"));
+      }
+      // special cases for MySQL
+      else if ( mDatabase.driverName().startsWith( QLatin1String("QMYSQL") ) ) {
         if ( entry.second.contains( QLatin1String("AUTOINCREMENT") ) )
           entry.second.replace(QLatin1String("AUTOINCREMENT"), QLatin1String("AUTO_INCREMENT"));
         if ( entry.second.startsWith( QLatin1String("CHAR") ) )
           entry.second.replace(QLatin1String("CHAR"), QLatin1String("VARCHAR"));
       }
+
+      if ( !columnElement.attribute( QLatin1String( "refTable" ) ).isEmpty()
+        && !columnElement.attribute( QLatin1String( "refColumn" ) ).isEmpty() )
+      {
+        const QString refStmt = QString::fromLatin1( " REFERENCES %1Table(%2) ON DELETE CASCADE ON UPDATE CASCADE" )
+          .arg( columnElement.attribute( QLatin1String( "refTable" ) ) )
+          .arg( columnElement.attribute( QLatin1String( "refColumn" ) ) );
+        if ( !mDatabase.driverName().startsWith( QLatin1String("QSQLITE") ) )
+          entry.second += refStmt;
+      }
+
       columnsList.append( entry );
     } else if ( columnElement.tagName() == QLatin1String( "data" ) ) {
       QString values = columnElement.attribute( QLatin1String("values") );
@@ -159,7 +191,8 @@ bool DbInitializer::checkTable( const QDomElement &element )
     qDebug() << statement;
 
     if ( !query.exec( statement ) ) {
-      mErrorMsg = QLatin1String( "Unable to create entire table." );
+      mErrorMsg = QLatin1String( "Unable to create entire table.\n" );
+      mErrorMsg += QString::fromLatin1( "Query error: '%1'" ).arg( query.lastError().text() );
       return false;
     }
   } else {
@@ -189,7 +222,8 @@ bool DbInitializer::checkTable( const QDomElement &element )
                                          .arg( tableName, entry.first, entry.second );
 
         if ( !query.exec( statement ) ) {
-          mErrorMsg = QString::fromLatin1( "Unable to add column '%1' to table '%2'." ).arg( entry.first, tableName );
+          mErrorMsg = QString::fromLatin1( "Unable to add column '%1' to table '%2'.\n" ).arg( entry.first, tableName );
+          mErrorMsg += QString::fromLatin1( "Query error: '%1'" ).arg( query.lastError().text() );
           return false;
         }
       }
@@ -202,12 +236,13 @@ bool DbInitializer::checkTable( const QDomElement &element )
   columnElement = element.firstChildElement();
   while ( !columnElement.isNull() ) {
     if ( columnElement.tagName() == QLatin1String( "index" ) ) {
-      if ( !hasIndex( tableName, columnElement.attribute( QLatin1String("name") ) ) ) {
+      QString indexName = QString::fromLatin1( "%1_%2" ).arg( tableName ).arg( columnElement.attribute( QLatin1String("name") ) ); // sqlite3 needs unique index identifiers per db
+      if ( !hasIndex( tableName, indexName ) ) {
         QString statement = QLatin1String( "CREATE " );
         if ( columnElement.attribute( QLatin1String("unique") ) == QLatin1String( "true" ) )
           statement += QLatin1String( "UNIQUE " );
         statement += QLatin1String( "INDEX " );
-        statement += columnElement.attribute( QLatin1String("name") );
+        statement += indexName;
         statement += QLatin1String( " ON " );
         statement += tableName;
         statement += QLatin1String( " (" );
@@ -216,7 +251,8 @@ bool DbInitializer::checkTable( const QDomElement &element )
         QSqlQuery query( mDatabase );
         qDebug() << "adding index" << statement;
         if ( !query.exec( statement ) ) {
-          mErrorMsg = QLatin1String( "Unable to create index." );
+          mErrorMsg = QLatin1String( "Unable to create index.\n" );
+          mErrorMsg += QString::fromLatin1( "Query error: '%1'" ).arg( query.lastError().text() );
           return false;
         }
       }
@@ -228,13 +264,15 @@ bool DbInitializer::checkTable( const QDomElement &element )
   // add initial data if table is empty
   const QString statement = QString::fromLatin1( "SELECT * FROM %1 LIMIT 1" ).arg( tableName );
   if ( !query.exec( statement ) ) {
-    mErrorMsg = QString::fromLatin1( "Unable to retrieve data from table '%1'." ).arg( tableName );
+    mErrorMsg = QString::fromLatin1( "Unable to retrieve data from table '%1'.\n" ).arg( tableName );
+    mErrorMsg += QString::fromLatin1( "Query error: '%1'" ).arg( query.lastError().text() );
     return false;
   }
   if ( query.size() == 0  || !query.first() ) {
     foreach ( const QString &stmt, dataList ) {
       if ( !query.exec( stmt ) ) {
-        mErrorMsg = QString::fromLatin1( "Unable to add initial data to table '%1'." ).arg( tableName );
+        mErrorMsg = QString::fromLatin1( "Unable to add initial data to table '%1'.\n" ).arg( tableName );
+        mErrorMsg += QString::fromLatin1( "Query error: '%1'\n" ).arg( query.lastError().text() );
         mErrorMsg += QString::fromLatin1( "Query was: %1" ).arg( stmt );
         return false;
       }
@@ -246,51 +284,45 @@ bool DbInitializer::checkTable( const QDomElement &element )
 
 bool DbInitializer::checkRelation(const QDomElement & element)
 {
-  QString table1 = element.attribute(QLatin1String("table1"));
-  QString table2 = element.attribute(QLatin1String("table2"));
-  QString col1 = element.attribute(QLatin1String("column1"));
-  QString col2 = element.attribute(QLatin1String("column2"));
+  const QString table1 = element.attribute(QLatin1String("table1"));
+  const QString table1Name = table1 + QLatin1String( "Table" );
+  const QString table2 = element.attribute(QLatin1String("table2"));
+  const QString table2Name = table2 + QLatin1String ( "Table" );
+  const QString col1 = element.attribute(QLatin1String("column1"));
+  const QString col2 = element.attribute(QLatin1String("column2"));
 
   QString tableName = table1 + table2 + QLatin1String("Relation");
   qDebug() << "checking relation " << tableName;
 
+  QString columnOptions;
+  // ### for MySQL as well?
+  if ( mDatabase.driverName() == QLatin1String( "QPSQL" ) )
+    columnOptions = QLatin1String( " ON DELETE CASCADE ON UPDATE CASCADE " );
+
   if ( !hasTable( tableName ) ) {
     QString statement = QString::fromLatin1( "CREATE TABLE %1 (" ).arg( tableName );
-    statement += QString::fromLatin1("%1_%2 INTEGER REFERENCES %3(%4), " )
-        .arg( table1 )
+    statement += QString::fromLatin1("%1_%2 INTEGER REFERENCES %3(%4) %5, " )
+        .arg( table1  )
         .arg( col1 )
-        .arg( table1 )
-        .arg( col1 );
-    statement += QString::fromLatin1("%1_%2 INTEGER REFERENCES %3(%4))" )
+        .arg( table1Name )
+        .arg( col1 )
+        .arg( columnOptions );
+    statement += QString::fromLatin1("%1_%2 INTEGER REFERENCES %3(%4) %5, " )
         .arg( table2 )
         .arg( col2 )
-        .arg( table2 )
-        .arg( col2 );
+        .arg( table2Name )
+        .arg( col2 )
+        .arg( columnOptions ),
+    statement += QString::fromLatin1("PRIMARY KEY (%1_%2, %3_%4));" ).arg( table1 ).arg( col1 ).arg( table2 ).arg( col2 );
     qDebug() << statement;
 
     QSqlQuery query( mDatabase );
     if ( !query.exec( statement ) ) {
-      mErrorMsg = QLatin1String( "Unable to create entire table." );
+      mErrorMsg = QLatin1String( "Unable to create entire table.\n" );
+      mErrorMsg += QString::fromLatin1( "Query error: '%1'" ).arg( query.lastError().text() );
       return false;
     }
   }
-
-  if ( !hasIndex( tableName, QLatin1String("PRIMARY") ) ) {
-    QString statement = QLatin1String( "ALTER TABLE " );
-    statement += tableName;
-    statement += QLatin1String( " ADD PRIMARY KEY (" );
-    statement += QString::fromLatin1( "%1_%2" ).arg( table1 ).arg( col1 );
-    statement += QLatin1String( ", " );
-    statement += QString::fromLatin1( "%1_%2" ).arg( table2 ).arg( col2 );
-    statement += QLatin1Char(')');
-    QSqlQuery query( mDatabase );
-    qDebug() << "adding index" << statement;
-    if ( !query.exec( statement ) ) {
-      mErrorMsg = QLatin1String( "Unable to create index." );
-      return false;
-    }
-  }
-
   return true;
 }
 
@@ -303,8 +335,11 @@ QString DbInitializer::sqlType(const QString & type)
 {
   if ( type == QLatin1String("int") )
     return QLatin1String("INTEGER");
-  if ( type == QLatin1String("qint64") )
-    return QLatin1String("BIGINT");
+  if ( type == QLatin1String("qint64") ) {
+    return ( mDatabase.driverName() == QLatin1String( "QPSQL" ) )
+            ? QLatin1String( "int8" )
+            : QLatin1String( "BIGINT" );
+  }
   if ( type == QLatin1String("QString") )
     return QLatin1String("TEXT");
   if (type == QLatin1String("QByteArray") )
@@ -321,27 +356,7 @@ QString DbInitializer::sqlType(const QString & type)
 
 bool DbInitializer::hasTable(const QString & tableName)
 {
-  QString statement;
-  if ( mDatabase.driverName() == QLatin1String( "QPSQL" ) )
-    statement = QLatin1String("SELECT tablename FROM pg_tables ORDER BY tablename;" );
-  else if ( mDatabase.driverName() == QLatin1String( "QSQLITE" ) )
-    statement = QLatin1String("SELECT name FROM sqlite_master WHERE type='table' ORDER BY name;");
-  else if ( mDatabase.driverName().startsWith( QLatin1String("QMYSQL") ) )
-    statement = QLatin1String("SHOW TABLES");
-  else
-    Q_ASSERT( false ); // unknown driver
-
-  QSqlQuery query( mDatabase );
-  if ( !query.exec( statement ) ) {
-    mErrorMsg = QLatin1String( "Unable to retrieve table information from database." );
-    return false;
-  }
-
-  while ( query.next() ) {
-    if ( query.value( 0 ).toString().toLower() == tableName.toLower() )
-      return true;
-  }
-  return false;
+   return mDatabase.tables().contains( tableName, Qt::CaseInsensitive );
 }
 
 bool DbInitializer::hasIndex(const QString & tableName, const QString & indexName)
@@ -351,10 +366,12 @@ bool DbInitializer::hasIndex(const QString & tableName, const QString & indexNam
   if ( mDatabase.driverName().startsWith( QLatin1String("QMYSQL") ) ) {
     statement  = QString::fromLatin1( "SHOW INDEXES FROM %1" ).arg( tableName );
     statement += QString::fromLatin1( " WHERE `Key_name` = '%1'" ).arg( indexName );
-  } else if( mDatabase.driverName() == QLatin1String("PSQL") ) {
-    statement  = QLatin1String( "SELECT indexname FROM pq_indexes" );
-    statement += QString::fromLatin1( " WHERE tablename = '%1'" ).arg( tableName );
-    statement += QString::fromLatin1( " AND  indexname = '%1';" ).arg( indexName );
+  } else if( mDatabase.driverName() == QLatin1String("QPSQL") ) {
+    statement  = QLatin1String( "SELECT indexname FROM pg_catalog.pg_indexes" );
+    statement += QString::fromLatin1( " WHERE tablename ilike '%1'" ).arg( tableName );
+    statement += QString::fromLatin1( " AND  indexname ilike '%1';" ).arg( indexName );
+  } else if ( mDatabase.driverName() == QLatin1String("QSQLITE") ) {
+    statement  = QString::fromLatin1( "SELECT * FROM sqlite_master WHERE type='index' AND tbl_name='%1' AND name='%2';" ).arg( tableName ).arg( indexName );
   } else {
     qFatal( "Implement index support for your database!" );
   }
@@ -362,7 +379,8 @@ bool DbInitializer::hasIndex(const QString & tableName, const QString & indexNam
   // query it
   QSqlQuery query( mDatabase );
   if ( !query.exec( statement ) ) {
-    mErrorMsg = QString::fromLatin1( "Unable to list index information for table %1." ).arg( tableName );
+    mErrorMsg = QString::fromLatin1( "Unable to list index information for table %1.\n" ).arg( tableName );
+    mErrorMsg += QString::fromLatin1( "Query error: '%1'" ).arg( query.lastError().text() );
     return false;
   }
   return query.next();
